@@ -4,9 +4,14 @@ import re
 from enum import Enum
 from typing import Any, Dict, List, NamedTuple, Optional, cast
 
-from pydantic import BaseModel, Field, root_validator
+try:
+    from pydantic.v1 import BaseModel, Field, root_validator
+except ImportError:
+    from pydantic import BaseModel, Field, root_validator
 
+from .deviceconfig import DeviceConfig
 from .modules import Antitheft, Cloud, Countdown, Emeter, Schedule, Time, Usage
+from .protocol import BaseProtocol
 from .smartdevice import DeviceType, SmartDevice, SmartDeviceException, requires_update
 
 
@@ -30,9 +35,16 @@ class SmartBulbPreset(BaseModel):
 
     index: int
     brightness: int
-    hue: int
-    saturation: int
-    color_temp: int
+
+    # These are not available for effect mode presets on light strips
+    hue: Optional[int]
+    saturation: Optional[int]
+    color_temp: Optional[int]
+
+    # Variables for effect mode presets
+    custom: Optional[int]
+    id: Optional[str]
+    mode: Optional[int]
 
 
 class BehaviorMode(str, Enum):
@@ -48,7 +60,8 @@ class TurnOnBehavior(BaseModel):
     """Model to present a single turn on behavior.
 
     :param int preset: the index number of wanted preset.
-    :param BehaviorMode mode: last status or preset mode. If you are changing existing settings, you should not set this manually.
+    :param BehaviorMode mode: last status or preset mode.
+     If you are changing existing settings, you should not set this manually.
 
     To change the behavior, it is only necessary to change the :attr:`preset` field
     to contain either the preset index, or ``None`` for the last known state.
@@ -110,9 +123,11 @@ class SmartBulb(SmartDevice):
     This will allow accessing the properties using the exposed properties.
 
     All changes to the device are done using awaitable methods,
-    which will not change the cached values, but you must await :func:`update()` separately.
+    which will not change the cached values,
+    so you must await :func:`update()` to fetch updates values from the device.
 
-    Errors reported by the device are raised as :class:`SmartDeviceExceptions <kasa.exceptions.SmartDeviceException>`,
+    Errors reported by the device are raised as
+    :class:`SmartDeviceExceptions <kasa.exceptions.SmartDeviceException>`,
     and should be handled by the user of the library.
 
     Examples:
@@ -120,7 +135,7 @@ class SmartBulb(SmartDevice):
         >>> bulb = SmartBulb("127.0.0.1")
         >>> asyncio.run(bulb.update())
         >>> print(bulb.alias)
-        KL130 office bulb
+        Bulb2
 
         Bulbs, like any other supported devices, can be turned on and off:
 
@@ -148,7 +163,7 @@ class SmartBulb(SmartDevice):
         >>> bulb.brightness
         50
 
-        Bulbs supporting color temperature can be queried to know which range is accepted:
+        Bulbs supporting color temperature can be queried for the supported range:
 
         >>> bulb.valid_temperature_range
         ColorTempRange(min=2500, max=9000)
@@ -164,9 +179,18 @@ class SmartBulb(SmartDevice):
         >>> bulb.hsv
         HSV(hue=180, saturation=100, value=80)
 
-        If you don't want to use the default transitions, you can pass `transition` in milliseconds.
-        This applies to all transitions (:func:`turn_on`, :func:`turn_off`, :func:`set_hsv`, :func:`set_color_temp`, :func:`set_brightness`) if supported by the device.
-        Light strips (e.g., KL420L5) do not support this feature, but silently ignore the parameter.
+        If you don't want to use the default transitions,
+        you can pass `transition` in milliseconds.
+        All methods changing the state of the device support this parameter:
+
+        * :func:`turn_on`
+        * :func:`turn_off`
+        * :func:`set_hsv`
+        * :func:`set_color_temp`
+        * :func:`set_brightness`
+
+        Light strips (e.g., KL420L5) do not support this feature,
+        but silently ignore the parameter.
         The following changes the brightness over a period of 10 seconds:
 
         >>> asyncio.run(bulb.set_brightness(100, transition=10_000))
@@ -174,9 +198,10 @@ class SmartBulb(SmartDevice):
         Bulb configuration presets can be accessed using the :func:`presets` property:
 
         >>> bulb.presets
-        [SmartBulbPreset(index=0, brightness=50, hue=0, saturation=0, color_temp=2700), SmartBulbPreset(index=1, brightness=100, hue=0, saturation=75, color_temp=0), SmartBulbPreset(index=2, brightness=100, hue=120, saturation=75, color_temp=0), SmartBulbPreset(index=3, brightness=100, hue=240, saturation=75, color_temp=0)]
+        [SmartBulbPreset(index=0, brightness=50, hue=0, saturation=0, color_temp=2700, custom=None, id=None, mode=None), SmartBulbPreset(index=1, brightness=100, hue=0, saturation=75, color_temp=0, custom=None, id=None, mode=None), SmartBulbPreset(index=2, brightness=100, hue=120, saturation=75, color_temp=0, custom=None, id=None, mode=None), SmartBulbPreset(index=3, brightness=100, hue=240, saturation=75, color_temp=0, custom=None, id=None, mode=None)]
 
-        To modify an existing preset, pass :class:`~kasa.smartbulb.SmartBulbPreset` instance to :func:`save_preset` method:
+        To modify an existing preset, pass :class:`~kasa.smartbulb.SmartBulbPreset`
+        instance to :func:`save_preset` method:
 
         >>> preset = bulb.presets[0]
         >>> preset.brightness
@@ -186,14 +211,20 @@ class SmartBulb(SmartDevice):
         >>> bulb.presets[0].brightness
         100
 
-    """
+    """  # noqa: E501
 
     LIGHT_SERVICE = "smartlife.iot.smartbulb.lightingservice"
     SET_LIGHT_METHOD = "transition_light_state"
     emeter_type = "smartlife.iot.common.emeter"
 
-    def __init__(self, host: str) -> None:
-        super().__init__(host=host)
+    def __init__(
+        self,
+        host: str,
+        *,
+        config: Optional[DeviceConfig] = None,
+        protocol: Optional[BaseProtocol] = None,
+    ) -> None:
+        super().__init__(host=host, config=config, protocol=protocol)
         self._device_type = DeviceType.Bulb
         self.add_module("schedule", Schedule(self, "smartlife.iot.common.schedule"))
         self.add_module("usage", Usage(self, "smartlife.iot.common.schedule"))
@@ -344,9 +375,7 @@ class SmartBulb(SmartDevice):
 
     def _raise_for_invalid_brightness(self, value):
         if not isinstance(value, int) or not (0 <= value <= 100):
-            raise ValueError(
-                "Invalid brightness value: {} " "(valid range: 0-100%)".format(value)
-            )
+            raise ValueError(f"Invalid brightness value: {value} (valid range: 0-100%)")
 
     @requires_update
     async def set_hsv(
@@ -355,7 +384,7 @@ class SmartBulb(SmartDevice):
         saturation: int,
         value: Optional[int] = None,
         *,
-        transition: Optional[int] = None
+        transition: Optional[int] = None,
     ) -> Dict:
         """Set new HSV.
 
@@ -368,14 +397,11 @@ class SmartBulb(SmartDevice):
             raise SmartDeviceException("Bulb does not support color.")
 
         if not isinstance(hue, int) or not (0 <= hue <= 360):
-            raise ValueError(
-                "Invalid hue value: {} " "(valid range: 0-360)".format(hue)
-            )
+            raise ValueError(f"Invalid hue value: {hue} (valid range: 0-360)")
 
         if not isinstance(saturation, int) or not (0 <= saturation <= 100):
             raise ValueError(
-                "Invalid saturation value: {} "
-                "(valid range: 0-100%)".format(saturation)
+                f"Invalid saturation value: {saturation} (valid range: 0-100%)"
             )
 
         light_state = {
@@ -415,8 +441,9 @@ class SmartBulb(SmartDevice):
         valid_temperature_range = self.valid_temperature_range
         if temp < valid_temperature_range[0] or temp > valid_temperature_range[1]:
             raise ValueError(
-                "Temperature should be between {} "
-                "and {}, was {}".format(*valid_temperature_range, temp)
+                "Temperature should be between {} and {}, was {}".format(
+                    *valid_temperature_range, temp
+                )
             )
 
         light_state = {"color_temp": temp}
@@ -514,7 +541,7 @@ class SmartBulb(SmartDevice):
     async def save_preset(self, preset: SmartBulbPreset):
         """Save a setting preset.
 
-        You can either construct a preset object manually, or pass an existing one obtained
+        You can either construct a preset object manually, or pass an existing one
         obtained using :func:`presets`.
         """
         if len(self.presets) == 0:
@@ -524,5 +551,10 @@ class SmartBulb(SmartDevice):
             raise SmartDeviceException("Invalid preset index")
 
         return await self._query_helper(
-            self.LIGHT_SERVICE, "set_preferred_state", preset.dict()
+            self.LIGHT_SERVICE, "set_preferred_state", preset.dict(exclude_none=True)
         )
+
+    @property
+    def max_device_response_size(self) -> int:
+        """Returns the maximum response size the device can safely construct."""
+        return 4096
