@@ -5,25 +5,30 @@ import os
 from dataclasses import dataclass
 from json import dumps as json_dumps
 from os.path import basename
-from pathlib import Path, PurePath
-from typing import Dict, Optional, Set
+from pathlib import Path
+from typing import Dict, Optional
 from unittest.mock import MagicMock
 
 import pytest  # type: ignore # see https://github.com/pytest-dev/pytest/issues/3342
 
 from kasa import (
     Credentials,
+    DeviceConfig,
     Discover,
     SmartBulb,
+    SmartDevice,
     SmartDimmer,
     SmartLightStrip,
     SmartPlug,
+    SmartProtocol,
     SmartStrip,
-    TPLinkSmartHomeProtocol,
 )
-from kasa.tapo import TapoBulb, TapoDevice, TapoPlug
+from kasa.protocol import BaseTransport
+from kasa.tapo import TapoBulb, TapoPlug
+from kasa.xortransport import XorEncryption
 
-from .newfakes import FakeSmartProtocol, FakeTransportProtocol
+from .fakeprotocol_iot import FakeIotProtocol
+from .fakeprotocol_smart import FakeSmartProtocol
 
 SUPPORTED_IOT_DEVICES = [
     (device, "IOT")
@@ -43,10 +48,10 @@ SUPPORTED_SMART_DEVICES = [
 SUPPORTED_DEVICES = SUPPORTED_IOT_DEVICES + SUPPORTED_SMART_DEVICES
 
 # Tapo bulbs
-BULBS_SMART_VARIABLE_TEMP = {"L530E"}
-BULBS_SMART_LIGHT_STRIP = {"L900-5", "L900-10", "L920-5"}
+BULBS_SMART_VARIABLE_TEMP = {"L530E", "L930-5"}
+BULBS_SMART_LIGHT_STRIP = {"L900-5", "L900-10", "L920-5", "L930-5"}
 BULBS_SMART_COLOR = {"L530E", *BULBS_SMART_LIGHT_STRIP}
-BULBS_SMART_DIMMABLE = {"KS225"}
+BULBS_SMART_DIMMABLE = {"KS225", "L510B", "L510E"}
 BULBS_SMART = (
     BULBS_SMART_VARIABLE_TEMP.union(BULBS_SMART_COLOR)
     .union(BULBS_SMART_DIMMABLE)
@@ -98,17 +103,29 @@ PLUGS_IOT = {
     "KP401",
     "KS200M",
 }
-PLUGS_SMART = {"P110", "KP125M", "EP25", "KS205", "P125M"}
+# P135 supports dimming, but its not currently support
+# by the library
+PLUGS_SMART = {
+    "P100",
+    "P110",
+    "KP125M",
+    "EP25",
+    "KS205",
+    "P125M",
+    "P135",
+    "S505",
+    "TP15",
+}
 PLUGS = {
     *PLUGS_IOT,
     *PLUGS_SMART,
 }
 STRIPS_IOT = {"HS107", "HS300", "KP303", "KP200", "KP400", "EP40"}
-STRIPS_SMART: Set[str] = set()
+STRIPS_SMART = {"P300", "TP25"}
 STRIPS = {*STRIPS_IOT, *STRIPS_SMART}
 
 DIMMERS_IOT = {"ES20M", "HS220", "KS220M", "KS230", "KP405"}
-DIMMERS_SMART: Set[str] = set()
+DIMMERS_SMART = {"S500D"}
 DIMMERS = {
     *DIMMERS_IOT,
     *DIMMERS_SMART,
@@ -207,7 +224,7 @@ no_emeter_iot = parametrize(
 
 bulb = parametrize("bulbs", BULBS, protocol_filter={"SMART", "IOT"})
 plug = parametrize("plugs", PLUGS, protocol_filter={"IOT"})
-strip = parametrize("strips", STRIPS, protocol_filter={"IOT"})
+strip = parametrize("strips", STRIPS, protocol_filter={"SMART", "IOT"})
 dimmer = parametrize("dimmers", DIMMERS, protocol_filter={"IOT"})
 lightstrip = parametrize("lightstrips", LIGHT_STRIPS, protocol_filter={"IOT"})
 
@@ -235,8 +252,16 @@ variable_temp_iot = parametrize(
 )
 bulb_iot = parametrize("bulb devices iot", BULBS_IOT, protocol_filter={"IOT"})
 
+strip_iot = parametrize("strip devices iot", STRIPS_IOT, protocol_filter={"IOT"})
+strip_smart = parametrize(
+    "strip devices smart", STRIPS_SMART, protocol_filter={"SMART"}
+)
+
 plug_smart = parametrize("plug devices smart", PLUGS_SMART, protocol_filter={"SMART"})
 bulb_smart = parametrize("bulb devices smart", BULBS_SMART, protocol_filter={"SMART"})
+dimmers_smart = parametrize(
+    "dimmer devices smart", DIMMERS_SMART, protocol_filter={"SMART"}
+)
 device_smart = parametrize(
     "devices smart", ALL_DEVICES_SMART, protocol_filter={"SMART"}
 )
@@ -297,6 +322,7 @@ def check_categories():
         + lightstrip.args[1]
         + plug_smart.args[1]
         + bulb_smart.args[1]
+        + dimmers_smart.args[1]
     )
     diff = set(SUPPORTED_DEVICES) - set(categorized_fixtures)
     if diff:
@@ -328,6 +354,12 @@ def device_for_file(model, protocol):
         for d in BULBS_SMART:
             if d in model:
                 return TapoBulb
+        for d in DIMMERS_SMART:
+            if d in model:
+                return TapoBulb
+        for d in STRIPS_SMART:
+            if d in model:
+                return TapoPlug
     else:
         for d in STRIPS_IOT:
             if d in model:
@@ -389,7 +421,7 @@ async def get_device_for_file(file, protocol):
     if protocol == "SMART":
         d.protocol = FakeSmartProtocol(sysinfo)
     else:
-        d.protocol = FakeTransportProtocol(sysinfo)
+        d.protocol = FakeIotProtocol(sysinfo)
     await _update_and_close(d)
     return d
 
@@ -414,9 +446,15 @@ async def dev(request):
             IP_MODEL_CACHE[ip] = model = d.model
         if model not in file:
             pytest.skip(f"skipping file {file}")
-        return d if d else await _discover_update_and_close(ip, username, password)
+        dev: SmartDevice = (
+            d if d else await _discover_update_and_close(ip, username, password)
+        )
+    else:
+        dev: SmartDevice = await get_device_for_file(file, protocol)
 
-    return await get_device_for_file(file, protocol)
+    yield dev
+
+    await dev.disconnect()
 
 
 @pytest.fixture
@@ -462,7 +500,7 @@ def discovery_mock(all_fixture_data, mocker):
         device_type = sys_info.get("mic_type") or sys_info.get("type")
         encrypt_type = "XOR"
         login_version = None
-        datagram = TPLinkSmartHomeProtocol.encrypt(json_dumps(discovery_data))[4:]
+        datagram = XorEncryption.encrypt(json_dumps(discovery_data))[4:]
         dm = _DiscoveryMock(
             "127.0.0.123",
             9999,
@@ -494,14 +532,13 @@ def discovery_mock(all_fixture_data, mocker):
     if "component_nego" in dm.query_data:
         proto = FakeSmartProtocol(dm.query_data)
     else:
-        proto = FakeTransportProtocol(dm.query_data)
+        proto = FakeIotProtocol(dm.query_data)
 
     async def _query(request, retry_count: int = 3):
         return await proto.query(request)
 
     mocker.patch("kasa.IotProtocol.query", side_effect=_query)
     mocker.patch("kasa.SmartProtocol.query", side_effect=_query)
-    mocker.patch("kasa.TPLinkSmartHomeProtocol.query", side_effect=_query)
 
     yield dm
 
@@ -541,6 +578,34 @@ def unsupported_device_info(request, mocker):
     yield discovery_data
 
 
+@pytest.fixture()
+def dummy_protocol():
+    """Return a smart protocol instance with a mocking-ready dummy transport."""
+
+    class DummyTransport(BaseTransport):
+        @property
+        def default_port(self) -> int:
+            return -1
+
+        @property
+        def credentials_hash(self) -> str:
+            return "dummy hash"
+
+        async def send(self, request: str) -> Dict:
+            return {}
+
+        async def close(self) -> None:
+            pass
+
+        async def reset(self) -> None:
+            pass
+
+    transport = DummyTransport(config=DeviceConfig(host="127.0.0.123"))
+    protocol = SmartProtocol(transport=transport)
+
+    return protocol
+
+
 def pytest_addoption(parser):
     parser.addoption(
         "--ip", action="store", default=None, help="run against device on given ip"
